@@ -19,7 +19,11 @@ from uuid import UUID
 from .util import fix_float_single_double_conversion
 
 if TYPE_CHECKING:
-    from .api_pb2 import BluetoothServiceData, HomeassistantServiceMap  # type: ignore
+    from .api_pb2 import (  # type: ignore
+        BluetoothLEAdvertisementResponse,
+        BluetoothServiceData,
+        HomeassistantServiceMap,
+    )
 
 # All fields in here should have defaults set
 # Home Assistant depends on these fields being constructible
@@ -815,22 +819,38 @@ def _convert_bluetooth_le_manufacturer_data(
     return {int(v.uuid, 16): bytes(v.data if v.data else v.legacy_data) for v in value}  # type: ignore
 
 
+def _convert_bluetooth_le_name(value: bytes) -> str:
+    return value.decode("utf-8", errors="replace")
+
+
 @dataclass(frozen=True)
 class BluetoothLEAdvertisement(APIModelBase):
-    address: int = 0
-    name: str = ""
-    rssi: int = 0
-    address_type: int = 0
+    def __post_init__(self) -> None:
+        """Post init hook disabled."""
 
-    service_uuids: List[str] = converter_field(
-        default_factory=list, converter=_convert_bluetooth_le_service_uuids
-    )
-    service_data: Dict[str, bytes] = converter_field(
-        default_factory=dict, converter=_convert_bluetooth_le_service_data
-    )
-    manufacturer_data: Dict[int, bytes] = converter_field(
-        default_factory=dict, converter=_convert_bluetooth_le_manufacturer_data
-    )
+    address: int
+    rssi: int
+    address_type: int
+    name: str
+    service_uuids: List[str]
+    service_data: Dict[str, bytes]
+    manufacturer_data: Dict[int, bytes]
+
+    @classmethod
+    def from_pb(  # type: ignore[misc]
+        cls: "BluetoothLEAdvertisement", data: "BluetoothLEAdvertisementResponse"
+    ) -> "BluetoothLEAdvertisement":
+        return cls(  # type: ignore[operator, no-any-return]
+            address=data.address,
+            rssi=data.rssi,
+            address_type=data.address_type,
+            name=_convert_bluetooth_le_name(data.name),
+            service_uuids=_convert_bluetooth_le_service_uuids(data.service_uuids),
+            service_data=_convert_bluetooth_le_service_data(data.service_data),
+            manufacturer_data=_convert_bluetooth_le_manufacturer_data(
+                data.manufacturer_data
+            ),
+        )
 
 
 @dataclass(frozen=True)
